//+------------------------------------------------------------------+
//|                                                  LeManSignal.mqh |
//|                             Copyright  2011,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
#property tester_indicator "LeManSignal.ex5"
#include <Expert\ExpertSignal.mqh>
//--- wizard description start
//+------------------------------------------------------------------+ 
//|                                                |
//+------------------------------------------------------------------+ 
#define OPEN_LONG     80  //          
#define OPEN_SHORT    80  //          
#define CLOSE_LONG    40  //          
#define CLOSE_SHORT   40  //          
#define REVERSE_LONG  100 //         
#define REVERSE_SHORT 100 //         
#define NO_SIGNAL      0  //         
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=,    LeManSignal              |
//| Type=SignalAdvanced                                              |
//| Name=LeManSignal                                                 |
//| Class=CLeManSignal                                               |
//| Page=                                                            |
//| Parameter=BuyPosOpen,bool,true,           |
//| Parameter=SellPosOpen,bool,true,          |
//| Parameter=BuyPosClose,bool,true,      |
//| Parameter=SellPosClose,bool,true,     |
//| Parameter=Ind_Timeframe,ENUM_TIMEFRAMES,PERIOD_H4,      |
//| Parameter=LPeriod,uint,12,                       |
//| Parameter=SignalBar,uint,1,              |
//+------------------------------------------------------------------+
//--- wizard description end
//+------------------------------------------------------------------+
//|  CLeManSignal.                                              |
//| :            |
//|  LeManSignal http://www.mql5.com/ru/code/474/.         |
//|     CExpertSignal.                    |
//+------------------------------------------------------------------+
class CLeManSignal : public CExpertSignal
  {
protected:
   CiCustom          m_indicator;        //      LeMan

   //---  
   bool              m_BuyPosOpen;       //     
   bool              m_SellPosOpen;      //     
   bool              m_BuyPosClose;      //     
   bool              m_SellPosClose;     //     
   ENUM_TIMEFRAMES   m_Ind_Timeframe;    //   LeMan
   uint              m_LPeriod;          //  
   uint              m_SignalBar;        //     

public:
                     CLeManSignal();

   //---    
   void               BuyPosOpen(bool value)                  { m_BuyPosOpen=value;       }
   void               SellPosOpen(bool value)                 { m_SellPosOpen=value;      }
   void               BuyPosClose(bool value)                 { m_BuyPosClose=value;      }
   void               SellPosClose(bool value)                { m_SellPosClose=value;     }
   void               Ind_Timeframe(ENUM_TIMEFRAMES value)    { m_Ind_Timeframe=value;    }
   void               LPeriod(uint value)                     { m_LPeriod=value;          }
   void               SignalBar(uint value)                   { m_SignalBar=value;        }

   //---    
   virtual bool      ValidationSettings();
   //---    
   virtual bool      InitIndicators(CIndicators *indicators); //  
   //---      
   virtual int       LongCondition();
   virtual int       ShortCondition();

   bool              InitLeMan(CIndicators *indicators);   // ,   LeManSignal

  };
//+------------------------------------------------------------------+
//|  CLeManSignal.                                        |
//| INPUT:  .                                                     |
//| OUTPUT: .                                                     |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
void CLeManSignal::CLeManSignal()
  {
//---    
   m_BuyPosOpen=true;
   m_SellPosOpen=true;
   m_BuyPosClose=true;
   m_SellPosClose=true;

//---   
   m_Ind_Timeframe=PERIOD_H4;
   m_LPeriod=12;
//---  
   m_SignalBar=1;
   m_used_series=USE_SERIES_OPEN+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE;
  }
//+------------------------------------------------------------------+
//|   .                                   |
//| INPUT:  .                                                     |
//| OUTPUT: true -   ,  false.           |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CLeManSignal::ValidationSettings()
  {
//---  
   if(m_LPeriod<=0)
     {
      printf(__FUNCTION__+":      ");
      return(false);
     }

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                           |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true -    ,  false.       |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CLeManSignal::InitIndicators(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---  
   if(!InitLeMan(indicators)) return(false);

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|   LeManSignal.                            |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true -    ,  false.       |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CLeManSignal::InitLeMan(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---    
   if(!indicators.Add(GetPointer(m_indicator)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---   
   MqlParam parameters[2];

   parameters[0].type=TYPE_STRING;
   parameters[0].string_value="LeManSignal.ex5";

   parameters[1].type=TYPE_UINT;
   parameters[1].integer_value=m_LPeriod;

//---     
   if(!m_indicator.Create(m_symbol.Name(),m_Ind_Timeframe,IND_CUSTOM,2,parameters))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---  
   if(!m_indicator.NumBuffers(2)) return(false);

//---     LeManSignal
   return(true);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CLeManSignal::LongCondition()
  {
//---       1  LeManSignal
   double Signal=m_indicator.GetData(1,m_SignalBar);

//---    
   if(Signal && Signal!=EMPTY_VALUE)
     {
      if(m_BuyPosOpen)
        {
         if(m_SellPosClose) return(REVERSE_SHORT);
         else return(OPEN_LONG);
        }
      else
        {
         if(m_SellPosClose) return(CLOSE_SHORT);
        }
     }

//---      
   if(!m_SellPosClose) return(NO_SIGNAL);

   int Bars_=Bars(m_symbol.Name(),m_Ind_Timeframe);

   for(int bar=int(m_SignalBar); bar<Bars_; bar++)
     {
      Signal=m_indicator.GetData(0,bar);
      if(Signal && Signal!=EMPTY_VALUE) return(NO_SIGNAL);

      Signal=m_indicator.GetData(1,bar);
      if(Signal && Signal!=EMPTY_VALUE) return(CLOSE_SHORT);
     }

//---   
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CLeManSignal::ShortCondition()
  {
//---       0  LeManSignal
   double Signal=m_indicator.GetData(0,m_SignalBar);

//---   
   if(Signal && Signal!=EMPTY_VALUE)
     {
      if(m_SellPosOpen)
        {
         if(m_BuyPosClose) return(REVERSE_LONG);
         else return(OPEN_SHORT);
        }
      else
        {
         if(m_BuyPosClose) return(CLOSE_LONG);
        }
     }

//---      
   if(!m_BuyPosClose) return(NO_SIGNAL);

   int Bars_=Bars(Symbol(),m_Ind_Timeframe);
   for(int bar=int(m_SignalBar); bar<Bars_; bar++)
     {
      Signal=m_indicator.GetData(1,bar);
      if(Signal && Signal!=EMPTY_VALUE) return(NO_SIGNAL);

      Signal=m_indicator.GetData(0,bar);
      if(Signal && Signal!=EMPTY_VALUE) return(CLOSE_LONG);
     }

//---      
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
